# -*- coding: utf8 -*-
from django.db import models
from django.db.models import F


class Revision(models.Model):
    """
    A blank model (except for ``id``) that is merely an implementation detail
    of django-revisionfield.
    """
    number = models.PositiveIntegerField()

    @staticmethod
    def next():
        """
        Returns the next revision.

        :returns: next available revision
        :rtype: ``int``
        """
        try:
            current = Revision.objects.get().number
        except Revision.DoesNotExist:
            revision, created = Revision.objects.get_or_create(number=1)
            current = revision.number

        while Revision.objects.filter(number=current).update(number=F('number') + 1) != 1:
            current = Revision.objects.get().number

        return current + 1



DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'tests.app',
    'django_revisionfield',
]


ROOT_URLCONF = 'tests.app.urls'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import os
os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.app.settings'

from attest import Tests
from django_attest import TestContext, FancyReporter
from django_revisionfield.models import Revision
from .app.models import Company, Person


loader = FancyReporter.test_loader
everything = Tests()
everything.context(TestContext())


@everything.test
def creating_model_instance_should_use_new_revision():
    current_revision = Revision.next()
    person = Person.objects.create(name="Brad")
    assert person.revision > current_revision


@everything.test
def saving_model_should_increment_revision():
    person = Person.objects.create(name="Brad")
    revision = person.revision
    person.name = "Sunny"
    person.save()
    assert person.revision > revision


@everything.test
def specifying_field_should_only_increment_revision_if_field_changes():
    company = Company.objects.create(name="brad pty ltd")
    name_revision = company.name_revision
    company.name = "chris pty ltd"
    company.save()
    assert company.name_revision > name_revision
    name_revision = company.name_revision
    company.address = "brisbane"
    company.save()
    assert company.name_revision == name_revision
    name_revision = company.name_revision
    company.save()
    assert name_revision == company.name_revision
    company.address = "sydney"
    company.save()
    assert name_revision == company.name_revision
    company.name = "sunny pty ltd"
    company.save()
    assert company.name_revision > name_revision


# -----------------------------------------------------------------------------


junit = Tests()

@junit.test
def make_junit_output():
    import xmlrunner
    runner = xmlrunner.XMLTestRunner(output=b"reports")
    runner.run(everything.test_suite())


# -----------------------------------------------------------------------------


pylint = Tests()

@pylint.test
def make_pylint_output():
    from os.path import expanduser
    from pylint.lint import Run
    from pylint.reporters.text import ParseableTextReporter
    if not os.path.exists('reports'):
        os.mkdir('reports')
    with open('reports/pylint.report', 'wb') as handle:
        args = ['django_revisionfield', 'tests']
        Run(args, reporter=ParseableTextReporter(output=handle), exit=False)



from django.db import models
from django_revisionfield import RevisionField


class Person(models.Model):
    name = models.CharField(max_length=200)
    revision = RevisionField()


class Company(models.Model):
    name = models.CharField(max_length=200)
    name_revision = RevisionField('name')
    address = models.CharField(max_length=200, blank=True)



from django.db import models
from .models import Revision


class RevisionField(models.IntegerField):
    __metaclass__ = models.SubfieldBase

    def __init__(self, watch=None, **kwargs):
        if isinstance(watch, basestring):
            watch = (watch, )
        self.watch = watch
        kwargs.setdefault('editable', False)
        kwargs.setdefault('unique', True)
        kwargs.setdefault('db_index', True)
        kwargs.setdefault('default', True)
        super(RevisionField, self).__init__(**kwargs)

    def pre_save(self, instance, add):
        """Returns field's value just before saving."""
        if self.watch and not add:
            old = instance.__class__.objects.get(pk=instance.pk)
            increment = any((getattr(old, w) != getattr(instance, w)
                             for w in self.watch))
        else:
            increment = True

        if increment:
            revision = Revision.next()
            setattr(instance, self.attname, revision)
            return revision
        else:
            return super(RevisionField, self).pre_save(instance, add)


try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules((
        (
            (RevisionField, ),
            (),
            {"watch": ("watch", {"default": None})},
        ),
    ), ("^django_revisionfield\.fields\.", ))



# -*- coding: utf8 -*-
from setuptools import setup, find_packages


setup(
    name='django-revisionfield',
    version='0.2.2',
    description = 'An model field that auto increments every time the model is'
                  ' saved',

    author='Bradley Ayers',
    author_email='bradley.ayers@gmail.com',
    license='Simplified BSD',
    url='https://github.com/bradleyayers/django-revisionfield/',

    packages=find_packages(),

    install_requires=['Django >=1.2'],
    tests_require=['Django >=1.2', 'Attest >=0.4', 'django-attest >=0.2.2',
                   'unittest-xml-reporting'],

    test_loader='tests:loader',
    test_suite='tests.everything',

    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
